/***************************************************************************
                          kpl.h  -  description                              
                             -------------------                                         

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sat Apr 24 15:14:00 MEST 1999
                                           
    copyright            : (C) 2000 by Werner Stille                         
    email                : stille@uni-freiburg.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#ifndef KPL_H
#define KPL_H 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <ktmainwindow.h>
#include "kplots.h"

class KAccel;
class KDNDDropZone;
class KplDoc;
class KplView;
class ItemDlg;

/**
  * Kpl application.
  * Provides the kpl top level main window with menubar, toolbar and statusbar.
  * @author Werner Stille
  */

class KplApp : public KTMainWindow
{
  Q_OBJECT

public:
  /** Constructor. Creates a kpl window. */
  KplApp(); 
  /** Destructor. */
  ~KplApp();
	/**
	 * Opens a file.
   * @param path absolute path of the file.
   * @return true for success
	 */
	bool openDocumentFile(const char* path);
	/**
	 * Returns a pointer to the current document connected to the KTMainWindow
	 * instance and is used by the View class to access the document object's
	 * methods.
   * @return pointer to the current document.
   */	
	KplDoc* getDocument() const; 	

protected:
  /** Inserts menus and menu entries. */
  void initMenuBar();
  /** Inserts toolbar items. */
  void initToolBar();
  /** Calculates precision for statusbar. */
  void calcStatusBar();
  /** Inserts statusbar items. */
  void initStatusBar();
  /** Creates and initialize new statusbar. */
  void newStatusBar();
	/**
   * initKeyAccel creates the keyboard accelerator items for the available slots and changes the menu accelerators.
   * @see KAccel
   */
	void initKeyAccel();
	/**
   * Initializes the document object of the main window that is connected to the view in initView().
   * @see initView();
   */
	void initDocument();
  /**
   * Creates the centerwidget of the KTMainWindow instance and sets it as the view.
   */
  void initView();
  /**
   * Saves the options.
   * @param saveAll save everything when true.
   */
  void saveOptions(bool saveAll);
  /** Reads the options on init. */
  void readOptions();
  /**
   * Enables/disables menu entries and toolbar items.
   * @param id ID of menu entry and toolbar item.
   * @param enable true enables, false disables item.
   */
  void enableCommand(int id, bool enable);
  /**
   * Toggles boolean options.
   * @param id ID of menu entry.
   */
  void toggleOption(int id);
  /**
   * Opens an existing URL or local file for reading.
   * @param type one of the following: UNKNOWN, DATAFILE, PLOTFILE.
   */
  void openRead(fileTypes type);
  /**
   * Saves file using current or new name.
   * @param current true for save under current name.
   * @return true when file is saved.
   */
  bool save(bool current);
  /**
   * Opens file for saving.
   * @param url pointer to QString receiving the chosen file URL.
   * @param filter file name filter for file dialog.
   * @return true when file can be saved.
   */
  bool getWriteURL(QString &url, const char *filter);
  /**
   * Warns at unsaved changes.
   * @return true when ready to close file.
   */
  virtual bool queryClose();
	/**
	 * Add an opened file to the recent file list and update recentFileMenu.
   * @param path absolute path of the file.
	 */
	void addRecentFile(const char* path);
  /** Overloaded for message box on last window exit. */
  virtual bool queryExit();
  /**
   * Saves the window properties for each open window during session end
   * to the session config file, including saving the currently
   * opened file by a temporary filename provided by KApplication.
   * @see KTMainWindow#saveProperties
   */
  virtual void saveProperties(KConfig *);
  /**
   * Reads the session config file and restores the application's state
   * including the last opened files and documents by reading the
   * temporary files saved by saveProperties().
   * @see KTMainWindow#readProperties
   */
  virtual void readProperties(KConfig *);
  /**
   * Generates PostScript file.
   * @param landscape true for landscape orientation.
   */
  void PSOutput(bool landscape);
  QPopupMenu *fileMenu, *viewMenu, *optionsMenu, *psMenu, *recentFilesMenu,
             *popupMenu;
  KplView *view;
  KplDoc *doc;
  ItemDlg *itemDlg;
  KConfig* config;
  KAccel *key_accel;
  QStrList recentFiles;
  QTimer *timer;
  bool plot;
  int mainHeight, mainWidth, tRefresh, xPrec, yPrec;
  double fz;

  friend KplView;

protected slots:
  /**
   * Selects statusbar help entry by menu or toolbar ID.
   * @param id ID of menu entry and toolbar item.
   */
  void statusCallback(int id);
  /** Creates a new plot in the current view. */
  void slotFileNew();
  /** Opens an existing plot file. */
  void slotFileOpenPlot();
  /** Opens an existing data file. */
  void slotFileOpenData();
	/** Opens a file from the recent files menu. */
	void slotFileOpenRecent(int id);
  /** Saves the current plot file. */
  void slotFileSave();
  /** Saves the current plot as a file with different name. */
  void slotFileSaveAs();
  /** Closes the current file. */
  void slotFileClose();
  /** Plots the current file. */
  void slotFilePlot();
  /** Plots the current file conditionally. */
  void slotPlotCond();
  /** Generates portrait PostScript file. */
  void slotFilePSPortrait();
  /** Generates landscape PostScript file. */
  void slotFilePSLandscape();
  /** Prints the current file. */
  void slotFilePrint();
  /** Opens a new kpl window. */
  void slotFileNewWindow();
  /** Closes the current window. */
  void slotFileCloseWindow();
  /** Exits the application. */
  void slotFileQuit();
  /** Edits plot items. */
  void slotEditItems();
  /** Increases magnification. */
  void slotViewZoomIn();
  /** Decreases magnification. */
  void slotViewZoomOut();
  /** Sets magnification. */
  void slotViewZoom();
  /** Reloads file. */
  void slotViewReload();
  /** Reloads file periodically. */
  void slotViewAutoReload();
  /** Toggles the visibility of the toolbar. */
  void slotOptionsToolBar();
  /** Toggles the visibility of the statusbar. */
  void slotOptionsStatusBar();
  /** Toggles the unsaved changes warning option. */
  void slotOptionsWarning();
  /** Toggles the autoplot option. */
  void slotOptionsAutoPlot();
  /** Toggles the show function source option. */
  void slotOptionsShowSource();
  /** Toggles the bounding box calculation option. */
  void slotOptionsBBox();
  /** Toggles the save-at-end option. */
  void slotOptionsSaveAtEnd();
  /** Displays the key bindings dialog. */
  void slotOptionsKeys();
  /** Displays the basic settings dialog. */
  void slotOptionsBasic();
  /** Displays the autoplot settings dialog. */
  void slotOptionsAuto();
  /** Displays the autofit settings dialog. */
  void slotOptionsAutoFit();
  /** Saves options. */
  void slotOptionsSave();
  /**
   * Changes the status message.
   * @param text pointer to new message text.
   */
  void slotStatusMsg(const char *text);
  /**
   * Changes the status message of the whole statusbar temporarily.
   * @param text pointer to temporary message text.
   */
  void slotStatusHelpMsg(const char *text);
  /**
   * Opens files from URL list.
   * @param list pointer to list with URLs.
   */
  void slotOpenList(QStrList *list);
#if (KDE_VERSION_MAJOR < 2) && (KDE_VERSION_MINOR < 2)
  /**
   * Handles KDE drop events.
   * @param zone pointer to the drop zone of the kpl main window.
   */
  void slotDropEvent(KDNDDropZone *zone);
#endif
  /** Updates titlebar, menus and plot. */
  void slotUpdate();
  /** Updates titlebar and menus. */
  void updWin();
  /**
   * Updates mouse position on statusbar.
   * @param x horizontal position of the mouse cursor.
   * @param y vertical position of the mouse cursor.
   */
  void updStatus(int x, int y);
  /** Clears mouse position information on statusbar. */
  void clearStatus();
  /** Shrinks window. */
  void shrink();
  /** Shows main popup menu. */
  void showPopupMenu();
};
 
#endif // KPL_H
