/***************************************************************************
                          kpldoc.cpp  -  description                              
                             -------------------                                         

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sat Apr 24 15:14:00 MEST 1999
                                           
    copyright            : (C) 2000 by Werner Stille                         
    email                : stille@uni-freiburg.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <math.h>
#include <qfileinfo.h>
#include <qdir.h>
#include <klocale.h>
#include <kapp.h>
#include <kurl.h>
#include <ksimpleconfig.h>

#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
  #include <kmessagebox.h>
  #include <kio_job.h>
  #include <kio_netaccess.h>
#else
  #include <kmsgbox.h>
  #include <kfm.h>
#endif

#include "kpldoc.h"
#include "fitdlg.h"
#include "frameitem.h"
#include "arrayitem.h"
#include "funitem.h"
#include "legenditem.h"
#include "utils.h"

KplDoc::KplDoc(QObject *parent) :
 QObject(parent)
{
  fType = UNKNOWN;
  itd.setAutoDelete(true);
}

KplDoc::~KplDoc()
{
  freeAllItems();
}

void KplDoc::newDocument()
{
  m_file = i18n("Untitled");
  freeAllItems();
  aut.nyLeg = 0;
}

bool KplDoc::saveDocument(const char* filename)
{
  FILE *f = fopen(filename, "w");
  if (f) {
    fprintf(f, "[Items]\nnitems=%i\n", itd.count());
    for (unsigned int i = 0; i < itd.count(); i++) {
      fprintf(f, "\n[Item %i]\n", i);
      itd.at(i)->writePlo(f);
    }
    fclose(f);
    m_modified = false;
  } else
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
    KMessageBox::sorry((QWidget *) parent(), i18n("Writing to this file not possible."));
#else
    KMsgBox::message((QWidget *) parent(), i18n("Sorry"),
                     i18n("Writing to this file not possible."),
                     KMsgBox::EXCLAMATION);
#endif
  return (f);
}

void KplDoc::setModified(bool modified)
{
  m_modified = modified;
}

bool KplDoc::isModified()
{
  return m_modified;
}

void KplDoc::setCurrentDir(const char *dir)
{
  m_dir = dir;
}

const QString& KplDoc::currentDir() const
{
  return m_dir;
}

void KplDoc::setCurrentFile(const char* file)
{
  m_file = file;
}

const QString& KplDoc::currentFile() const
{
  return m_file;
}

void KplDoc::freeAllItems()
{
  while (KplItem *item = itd.last()) {
    item->freeItem(this);
    itd.remove();
  }
  setModified(false);
}

void KplDoc::newItem(itemTypes ityp)
{
  switch (ityp) {
    case FRAMEITEM:
      itd.append(new FrameItem(&aut));
      break;
    case ARRAYITEM:
      itd.append(new ArrayItem(&aut));
      break;
    case FUNITEM:
      itd.append(new FunItem(&aut));
      break;
    case PARFUNITEM:
      itd.append(new ParFunItem(&aut));
      break;
    case LEGENDITEM:
      itd.append(new LegendItem(&aut));
  }
}

bool KplDoc::autoScale(FrameItem *fd, ArrayItem *ad)
{
  if ((ad->x) && (aut.ixAuto < ad->ncols) && (aut.iyAuto < ad->ncols)) {
    if (autoFile(&fd->xmi, &fd->xma, &fd->xtic, &fd->mticx, &fd->ndigx, &fd->iex,
                 &ad->fx, ad->x[aut.ixAuto], ad->nrows, aut.autoNorm, 0.2))
      return true;
    if (autoFile(&fd->ymi, &fd->yma, &fd->ytic, &fd->mticy, &fd->ndigy, &fd->iey,
                 &ad->fy, ad->x[aut.iyAuto], ad->nrows, aut.autoNorm,
                 0.2 * aut.xlAuto / aut.ylAuto))
      return true;
    if (aut.autohPath)
      strncpy(fd->sh, ad->path, sizeof(aut.shAuto) - 1);
    int minCol = ad->ncols - 1;
    ad->ix = QMIN(aut.ixAuto, minCol);
    ad->iy = QMIN(aut.iyAuto, minCol);
    ad->ie = QMIN(aut.ieAuto, minCol);
    ad->symb = aut.autoSymb;
    ad->istart = 0;
    ad->n = ad->nrows;
    ad->errbars = aut.autoErr & (ad->ie < ad->ncols);
    return false;
  }
  return true;
}

void KplDoc::autoFit(ArrayItem *ad, FunItem *fd)
{
  QList<ArrayItem> arr;
  QList<FunItem> fun;

  strcpy(fd->pathy, aut.fitPath);
  strcpy(fd->namey, aut.fitName);
  memcpy(fd->py, aut.pFit, NPMAX * sizeof(double));
  fd->symb = aut.fitSymb;
  fd->color = aut.fitColor;
  fd->tmin = ad->x[ad->ix][ad->istart];
  fd->tmax = ad->x[ad->ix][ad->istart + ad->n - 1];
  fd->fx = ad->fx;
  fd->fy = ad->fy;
  if (getFuncAddr(fd->pathy, fd->namey, &fd->hmody, &fd->fkty))
    return;
  int mode = 0;
  if (aut.fitShowDlg)
    mode |= SHOWDLG;
  if (aut.fitSavePar)
    mode |= SAVEPAR;
  if (aut.fitFollow)
    mode |= FOLLOW;
  arr.append(ad);
  fun.append(fd);
  FitDlg dlg((QWidget *) parent(), this, &arr, &fun, mode);
  dlg.slotFit();
  if (aut.fitShowDlg) {
    connect(&dlg, SIGNAL(modelChanged()), this, SIGNAL(modelChanged()));
    emit modelChanged();
    dlg.exec();
  } else
    dlg.slotApply();
  return;
}

bool KplDoc::openDocument(const char* filename, fileTypes type)
{
  QString fn;

  KURL u = KURL(filename);
  if (u.isMalformed())
    u = QDir::cleanDirPath(QDir::currentDirPath() + "/" + filename);
  if (u.isLocalFile())
    fn = u.path();
  else
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
    if (!KIONetAccess::download(u.url(), fn))
#else
    if (!KFM::download(u.url(), fn))
#endif
      return false;
  fType = type;
  QFileInfo fi(u.path());
  if (fType == UNKNOWN) {
    QString s = filename;
    fType = ((s.length() - s.findRev(".plo", -1, false)) == 4) ? PLOTFILE : DATAFILE;
  }
  if (fType == DATAFILE) {
    freeAllItems();
    newItem(FRAMEITEM);
    newItem(ARRAYITEM);
    if (aut.autoFit)
      newItem(FUNITEM);
    if (u.isLocalFile())
      strcpy(((ArrayItem *) itd.at(1))->path, fn);
    else
      strcpy(((ArrayItem *) itd.at(1))->path, u.url());
    int nrows = ((ArrayItem *) itd.at(1))->readFile();
    if (nrows) {
      setModified();
      if (autoScale((FrameItem *) itd.at(0), (ArrayItem *) itd.at(1)))
        nrows = 0;
    }
    if (!nrows) {
      newDocument();
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
      KMessageBox::error((QWidget *) parent(),
                         QString(i18n("while loading the file\n")) + filename);
#else
      KMsgBox::message((QWidget *) parent(), i18n("Error"),
                       QString(i18n("while loading the file\n")) + filename, KMsgBox::EXCLAMATION);
#endif
      emit modelChanged();
      if (!u.isLocalFile())
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
        KIONetAccess::removeTempFile(fn);
#else
        KFM::removeTempFile(fn);
#endif
      return false;
    }
  } else {
    QFileInfo f2(fn);
    if (f2.isReadable()) {
      const char itemName[6][11] = {"", "FRAMEITEM", "ARRAYITEM", "FUNITEM",
                                    "PARFUNITEM", "LEGENDITEM"};
      newDocument();
      KSimpleConfig plo(fn, true);
      plo.setGroup("Items");
      int nItems = QMAX(plo.readNumEntry("nitems"), 0);
      if (nItems) {
        for (int i = 0; i < nItems; i++) {
          QString s;
          s.sprintf("Item %i", i);
          plo.setGroup(s);
          s = plo.readEntry("Type");
          int iType;
          for (iType = FRAMEITEM; iType <= LEGENDITEM; iType++)
            if (s == itemName[iType])
              break;
          switch (iType) {
            case FRAMEITEM:
              itd.append(new FrameItem(&plo, &aut));
              break;
            case ARRAYITEM:
              itd.append(new ArrayItem(&plo, &aut));
              break;
            case FUNITEM:
              itd.append(new FunItem(&plo, &aut));
              break;
            case PARFUNITEM:
              itd.append(new ParFunItem(&plo, &aut));
              break;
            case LEGENDITEM:
              itd.append(new LegendItem(&plo, &aut));
          }
        }
      }
    } else {
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
      KMessageBox::error((QWidget *) parent(),
                         QString(i18n("while loading the file\n")) + filename);
#else
      KMsgBox::message((QWidget *) parent(), i18n("Error"),
                       QString(i18n("while loading the file\n")) + filename, KMsgBox::EXCLAMATION);
#endif
      return false;
    }
  }
  if (u.isLocalFile()) {
    m_file = fi.absFilePath();
    m_dir = fi.dirPath(true);
  } else {
    m_file = u.url();
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
    m_dir = u.protocol() + ":" + u.host() + u.directory();
    KIONetAccess::removeTempFile(fn);
#else
    m_dir = u.directoryURL();
    KFM::removeTempFile(fn);
#endif
  }
  if (fType == DATAFILE)
    if (aut.autoFit)
      autoFit((ArrayItem *) itd.at(1), (FunItem *) itd.at(2));
  emit modelChanged();
  return true;
}

void KplDoc::connectIOfinished()
{
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
  connect(kIO, SIGNAL(sigFinished()), this, SLOT(slotIOfinished()));
#else
  connect(kIO, SIGNAL(finished()), this, SLOT(slotIOfinished()));
#endif
}

void KplDoc::slotIOfinished()
{
  delete kIO;
  remove(QDir::homeDirPath() + "/kpltmp.tmp");
}
