/***************************************************************************
                          kpldoc.h  -  description                              
                             -------------------                                         

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sat Apr 24 15:14:00 MEST 1999
                                           
    copyright            : (C) 2000 by Werner Stille                         
    email                : stille@uni-freiburg.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#ifndef KPLDOC_H
#define KPLDOC_H 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

#include <stdio.h>
#include <qobject.h>
#include <qlist.h>
#include "kplots.h"

class KplItem;
class FrameItem;
class ArrayItem;
class FunItem;

#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
  class KIOJob;
#else
  class KFM;
#endif

/**
  * Kpl model class.
  * Provides methods for reading, writing and scaling of plot data.
  * @author Werner Stille
  */
class KplDoc : public QObject
{
  Q_OBJECT

public:
  /**
   * Constructor.
   * @param parent pointer to parent QObject.
   */
  KplDoc(QObject *parent);
  /** Destructor. */
  ~KplDoc();
	/** Initializes the document generally. */
	void newDocument();
  /**
   * Saves the document under filename.
   * @param filename file path.
   */
  bool saveDocument(const char* filename);
  /**
   * Sets the modified flag for the document after a modifying action on the
   * view connected to the document.
   * @param modified true for document modified after last save action.
   */
  void setModified(bool modified = true);
  /**
   * Returns if the document is modified or not. Use this to determine if your
   * document needs saving by the user on closing.
   * @return true for document modified after last save action.
   */
  bool isModified();
  /**
   * Sets the current directory path.
   * @param dir current directory path.
   */
  void setCurrentDir(const char *dir);
  /**
   * Returns the current path.
   * @return current directory path.
   */
  const QString& currentDir() const;
	/**
	 * Sets the absolute filename of the document.
   * @param file absolute file path.
	 */
	void setCurrentFile(const char* file);
	/**
	 * Returns the absolute filename of the document.
   * @return absolute file path.
	 */
	const QString& currentFile() const;
  /**
   * Appends new item to list and initializes structure.
   * @param ityp type of item. One of the following:
   * FRAMEITEM, ARRAYITEM, FUNITEM, PARFUNITEM, LEGENDITEM.
   */
  void newItem(itemTypes ityp);
  /**
   * Reads file and initializes plot items.
   * @param filename filename or URL.
   * @param type one of the following: UNKNOWN, DATAFILE, PLOTFILE.
   * @return true for success
   */
  bool openDocument(const char* filename, fileTypes type = UNKNOWN);
  /** Connects IO signal finished() to slot slotIOfinished(). */
  void connectIOfinished();
  fileTypes fType;
  QList<KplItem> itd;
  AutoStruct aut;
#if (KDE_VERSION_MAJOR > 1) || (KDE_VERSION_MINOR > 1)
  KIOJob *kIO;
#else
  KFM *kIO;
#endif

public slots:
  /** Deletes KFM object and temporary file for upload. */
  void slotIOfinished();

protected:
  /** Deallocates storage of all plot items. */
  void freeAllItems();
  /**
   * Sets array and frame item elements by autoscaling.
   * @param fd pointer to frame item.
   * @param ad pointer to array item.
   * @return true for error
   */
  bool autoScale(FrameItem *fd, ArrayItem *ad);
  /**
   * Sets function item elements for autofit.
   * @param ad pointer to array item.
   * @param fd pointer to function item.
   */
  void autoFit(ArrayItem *ad, FunItem *fd);
 	/** The modified flag of the current document. */
  bool m_modified;
  QString m_file;
  QString m_dir;

signals:
  /** Emitted when changes may require updating the menus and repainting the page view. */
  void modelChanged();
};

#endif // KPLDOC_H
